package wtalk;

/*
 * Copyright (C) 2002 Matthew Wronka <wronkm@rpi.edu>
 * This file is released under the GNU General Public License version 2,
 * a version of which should have been provided along with this file.
 *
 * You might also find a copy at:
 * http://dmc.sf.net/files/gpl.txt
 * http://www.fsf.org/copyleft/gpl.html
 */

import wtalk.protocols.*;

/**
 * Multi-protocol messaging library for Java.  Supported protocols 
 * 	are contaiend within <code>wtalk.protocols.&lt;protocolname&gt;</code>.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 20021125+
 */
public class WTalk {
	static private String WTALK_NAME= "WTalk";
	static private String WTALK_VERSION = "20030507";
	static private String WTALK_AUTHOR = "Matthew Wronka <wronkm@rpi.edu>";

	protected WTalkProtocol wtp=null;
	protected int iProtocol=0;
	protected String m_ConnectionName = null;

	/**
	 * Returns String containing name, version and author.
	 *
	 * @return Returns String containing name, version and author.
	 */
	static public String getBanner() { 
		return WTALK_NAME + " v" + WTALK_VERSION + " by " + WTALK_AUTHOR; 
	}

	/**
	 * Default constructor
	 * 
	 * @param name Connection name to be used by this instance of WTalk
	 */
	public WTalk(String name) {
		WTalkInit(name);
	}

	/**
	 * Overloaded constructor, creates WTalkProtocol 
	 *	based on paramater value.
	 * 
	 * @param name Connection name to be used by this instance of WTalk
	 * @param protocol Protocol WTalk will use, defined in Protocol 
	 * @see wtalk.Protocol
	 */
	public WTalk(String name, int protocol) {
		WTalkInit(name);
		setWTalkProtocol(protocol);
	}

	private void WTalkInit(String name) {
		System.err.println(getBanner());
		m_ConnectionName=name;
	}

	/**
	 * Sets the prtocool WTalk will use.
	 *
	 * @param protocol Protocol which should be a constant 
	 *	specified in <code>wtalk.Protocol</code>
	 * @see wtalk.Protocol
	 */
	public void setWTalkProtocol(int protocol) {
		int i = protocol;
		if(i<=0 || i>=Protocol.MAX) 
			return;

		wtp = Protocol.getProtocol(m_ConnectionName, i);

		if(wtp!=null) iProtocol=i;
		else iProtocol = 0;
	}

	/**
	 * Returns the WTalkProtocol object associated with WTalk.
	 *
	 * @return WTalkProtocol object associated with WTalk.
	 * @see wtalk.protocols.WTalkProtocol
	 */
	public WTalkProtocol getWTalkProtocol() {
		return wtp;
	}

	/**
	 * Returns the integer value representing the current WTalkProtocol.
	 * <p>
	 * wtalk.Protocol.UNDEF is returned if there is no active WTalkProtocol.
	 *
	 * @return Integer value representing the current WTalkProtocol. 
	 * @see wtalk.Protocol
	 */
	public int getProtocol() {
		return iProtocol;	
	}

	/**
	 * Returns the name of the connection using this instance of WTalk.
	 *
	 * @returns Name of the connection using this instance of WTalk.
	 */
	public String getConnectionName() {
		return m_ConnectionName;
	}

}
