package wtalk.protocols.pop;

import java.io.*;
import java.util.StringTokenizer;
import java.util.Properties;

import javax.mail.*;
import javax.mail.internet.*;

import wtalk.RMsgHandler;

class POP3Checker extends Thread{
	private String mPOPServer = "";
	private String mLogin = "";
	private String mPasswd = "";

	private RMsgHandler rmh=null;	// generic message handler
	private RMsgHandler rsh = null;	// handles system messages if !null
	private RMsgHandler rph = null;	// handles private messages if !null

	public POP3Checker(){
		rmh = null;
	}

	private String getEmail(String s) {
		StringTokenizer st=new StringTokenizer(s, "<>");
		String tok1 ;
		tok1=st.nextToken();
		if(st.hasMoreTokens()) 
			return st.nextToken();
		else
			return tok1;
	}

	private String getBody(Object p){
		boolean bPlain = false;
		try{
		Multipart mp = (Multipart)p;
		if(mp.getCount()==1){
			return mp.getBodyPart(0).toString();
		}

		do{
		bPlain=!bPlain;
		for (int i=0, n=mp.getCount(); i<n; i++){
			Part part = mp.getBodyPart(i);
			MimeBodyPart mbp = (MimeBodyPart) part;

			if(bPlain){
				if(mbp.isMimeType("text/plain")){
					return mbp.toString();
				}
			}else{
				if(mbp.isMimeType("text/html")){
					return mbp.toString();
				}
			}
		}
		}while(bPlain);
		}catch(Exception e){}

		return p.toString();
	}

	private void handlePrivateMessage(Message message) {
		try{
			String user="";
			String body="";
			if(message.getReplyTo() != null && !message.getReplyTo()[0].equals(""))
				user=getEmail(message.getReplyTo()[0].toString());
			else
				user=getEmail(message.getFrom()[0].toString());

			body = getBody(message.getContent());

			rph.handle(null, user, body); 
		}catch(IOException ioe) {
		}catch(Exception e){}
	}

	public void ReportTo(RMsgHandler r){
		rmh = r;
	}

	public void ReportSystemTo(RMsgHandler r){
		rsh = r;
	}

	public void ReportPrivateTo(RMsgHandler r){
		rph = r;
	}

	public void setPasswd(String pass) { mPasswd = pass; }
	public void setServer(String host) { mPOPServer = host; }
	public void setLogin(String login) { mLogin = login; }

	public void run(){
		Properties props = new Properties();
		Message message[] = null;
		Session session = Session.getInstance(props, null);
		int lastMsgs = 0;

		while(true) try{
			Store store = session.getStore("pop3");
			store.connect(mPOPServer, mLogin, mPasswd);
	
			Folder folder = store.getFolder("INBOX");
			folder.open(Folder.READ_ONLY);


			message = folder.getMessages();
			if(message==null || message.length <= 0) continue;
			if(lastMsgs==message.length) {
				System.err.println("No new messages.");
				continue;
			}

System.out.println("Got " + message.length + " messages.");
			for (int  n=message.length; lastMsgs<n; ++lastMsgs) {
				System.out.println(lastMsgs+": " + message[lastMsgs].getFrom()[0] + "\t" + message[lastMsgs].getSubject());
				handlePrivateMessage(message[lastMsgs]);
			}

			folder.close(false);
			store.close();
			try{ java.lang.Thread.sleep(5000); } catch (java.lang.InterruptedException ie) {}
		}catch(NoSuchProviderException e){ 
			System.err.println("Error with POP3--unsupported???");
			break; 
		} catch (MessagingException e) { 
			System.err.println("Messaging Exception.");
			try{ java.lang.Thread.sleep(30000); } catch (java.lang.InterruptedException ie) {}
			continue; 
		}
	}
}

